@@ -1,16 +1,17 @@
 
 ###################################################################################
 #
-#   Embperl - Copyright (c) 1997-1999 Gerald Richter / ECOS
+#   Embperl - Copyright (c) 1997-2000 Gerald Richter / ECOS
 #
 #   You may distribute under the terms of either the GNU General Public
 #   License or the Artistic License, as specified in the Perl README file.
-#   For use with Apache httpd and mod_perl, see also Apache copyright.
 #
 #   THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
 #   IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 #   WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 #
+#   $Id: Embperl.pm,v 1.136 2000/12/04 07:41:00 richter Exp $
+#
 ###################################################################################
 
 
@@ -66,18 +67,21 @@ use vars qw(
     $escmode
 
     $SessionMgnt
+    $DefaultIDLength
 
     $req_rec
 
     %http_headers_out
+
+    $pathsplit
+    $multiplicity
     ) ;
 
 
 @ISA = qw(Exporter DynaLoader);
 
 
-$VERSION = '1.2.1';
-
+$VERSION = '1.3.0';
 
 # HTML::Embperl cannot be bootstrapped in nonlazy mode except
 # under mod_perl, because its dependencies import symbols like ap_palloc
@@ -109,14 +113,16 @@ $DefaultLog = '/tmp/embperl.log' ;
 %filepack = () ;    # translate filename to packagename
 $packno   = 1 ;     # for assigning unique packagenames
 
+$multiplicity = Multiplicity () ;
+
 @cleanups = () ;    # packages which need a cleanup
 $LogOutputFileno = 0 ;
+$pathsplit = $^O eq 'MSWin32'?';':';|:' ;   # separators for path
 
 # setup constans
 
 use constant dbgAll                 => -1 ;
 use constant dbgAllCmds             => 1024 ;
-use constant dbgCacheDisable        => 32768 ;
 use constant dbgCmd                 => 8 ;
 use constant dbgDefEval             => 16384 ;
 use constant dbgEarlyHttpHeader     => 65536 ;
@@ -138,6 +144,8 @@ use constant dbgStd                 => 1 ;
 use constant dbgSession             => 0x200000 ;
 use constant dbgTab                 => 64 ;
 use constant dbgWatchScalar         => 131072 ;
+use constant dbgParse               => 0x1000000 ; # reserved for Embperl 2.x
+use constant dbgObjectSearch        => 0x2000000 ;
 
 use constant epIOCGI                => 1 ;
 use constant epIOMod_Perl           => 3 ;
@@ -148,6 +156,7 @@ use constant escHtml                => 1 ;
 use constant escNone                => 0 ;
 use constant escStd                 => 3 ;
 use constant escUrl                 => 2 ;
+use constant escEscape              => 4 ;
 
 
 use constant optDisableChdir            => 128 ;
@@ -158,6 +167,7 @@ use constant optDisableHtmlScan         => 512 ;
 use constant optDisableInputScan        => 1024 ;
 use constant optDisableMetaScan         => 4096 ;
 use constant optDisableTableScan        => 2048 ;
+use constant optDisableSelectScan       => 0x800000 ;
 use constant optDisableVarCleanup       => 1 ;
 use constant optEarlyHttpHeader         => 64 ;
 use constant optOpcodeMask              => 8 ;
@@ -315,21 +325,17 @@ if (defined ($ENV{MOD_PERL}))
     { 
     eval 'use Apache' ; # make sure Apache.pm is loaded (is not at server startup in mod_perl < 1.11)
     die "use Apache failed: $@" if ($@); 
-    #eval 'use Apache::Constants qw(:common &OPT_EXECCGI)' ;
-    eval 'use Apache::Constants qw(&OPT_EXECCGI &DECLINED &OK &FORBIDDEN)' ;
+    eval 'use Apache::Constants qw(&OPT_EXECCGI &DECLINED &OK &FORBIDDEN &NOT_FOUND) ;' ;
     die "use Apache::Constants failed: $@" if ($@); 
-    XS_Init (epIOMod_Perl, $DefaultLog, $DebugDefault) ;
-#    eval 'sub OK        { 0 ;   }' if (!defined (&OK)) ;
-    eval 'sub NOT_FOUND { 404 ; }' if (!defined (&NOT_FOUND)) ;
-#    eval 'sub FORBIDDEN { 403 ; }' if (!defined (&FORBIDDEN)) ;
 
+    XS_Init (epIOMod_Perl, $DefaultLog, $DebugDefault) ;
     }
 else
     {
     eval 'sub OK        { 0 ;   }' ;
     eval 'sub NOT_FOUND { 404 ; }' ;
     eval 'sub FORBIDDEN { 403 ; }' ;
-    eval 'sub DECLINED  { 403 ; }' ; # in non mod_perl environement, same as FORBIDDEN
+    eval 'sub DECLINED  { 403 ; }' ; # in non mod_perl environment, same as FORBIDDEN
 no strict ;
     XS_Init (epIOPerl, $DefaultLog, $DebugDefault) ;
 use strict ;
@@ -348,14 +354,16 @@ tie *OUT, 'HTML::Embperl::Out' ;
 $SessionMgnt = 0 ;
 if (defined ($ENV{EMBPERL_SESSION_CLASSES}))
     { # Apache::Session 1.xx
-    my ($os, $lm) = split /\s*,\s*|\s+/, $ENV{EMBPERL_SESSION_CLASSES} ;
+    my ($os, $lm, $ser, $gen) = split /\s*,\s*|\s+/, $ENV{EMBPERL_SESSION_CLASSES} ;
     if (!$os || !$lm)
         {
         warn "[$$]SES:  EMBPERL_SESSION_CLASSES must be set properly (is $ENV{EMBPERL_SESSION_CLASSES})" ;
         }
     else
         {
-	my @args  ;
+	$ser ||= 'Storable' ;
+	$gen ||= 'MD5' ;
+        my @args  ;
         if (defined ($ENV{EMBPERL_SESSION_ARGS}))
             {
 	    my @arglist = split /\s+/, $ENV{EMBPERL_SESSION_ARGS} ;
@@ -371,16 +379,34 @@ if (defined ($ENV{EMBPERL_SESSION_CLASSES}))
 	    @args,
             lazy	   => 1,
 	    create_unknown => 1,
-	    object_store   => $os,
-	    lock_manager   => $lm
 	    ) ;
+        
+        my $ver  ;
+        if ($Apache::Session::VERSION =~ /^1\.0\d$/)
+            {
+	    $sargs{object_store} = $os ;
+	    $sargs{lock_manager} = $lm ;
+            $ver = '1.0x' ;
+	    $DefaultIDLength = 16 ; 	
+            }
+        else
+            { # Apache::Session >= 1.50
+	    $sargs{Store} = $os ;
+	    $sargs{Lock} = $lm ;
+	    $sargs{Generate} = $gen ;
+	    $sargs{Serialize} = $ser ;
+            $ver = '>= 1.50' ;
+	    $DefaultIDLength = 32 ; 	
+            }
 
-        require HTML::Embperl::Session ; 
+        my $session_handler = $ENV{EMBPERL_SESSION_HANDLER_CLASS} || 'HTML::Embperl::Session' ; 
+        eval "require $session_handler" ; 
+        die $@ if ($@)  ;
 
-	tie %udat, 'HTML::Embperl::Session', undef, \%sargs ;
-	tie %mdat, 'HTML::Embperl::Session', undef, \%sargs ;
+	tie %mdat, $session_handler, undef, \%sargs ;
+	tie %udat, $session_handler, undef, {%sargs, recreate_id => 1} ;
 	$SessionMgnt = 2 ;
-	warn "[$$]SES:  Embperl Session management enabled (1.xx)\n" if ($ENV{MOD_PERL}) ;
+	warn "[$$]SES:  Embperl Session management enabled ($ver)\n" if ($ENV{MOD_PERL}) ;
         }
     }
 elsif (exists $INC{'Apache/Session.pm'})
@@ -388,38 +414,16 @@ elsif (exists $INC{'Apache/Session.pm'})
     if ($Apache::Session::VERSION =~ /^0\.17/)
         {
         # Apache::Session = 0.17
-	$SessionMgnt = 1 ;
-	tie %udat, 'Apache::Session', $ENV{EMBPERL_SESSION_CLASS} || 'Win32',
-		      undef, {not_lazy=>0, autocommit=>0, lifetime=>&Apache::Session::LIFETIME} ;
-	tie %mdat, 'Apache::Session', $ENV{EMBPERL_SESSION_CLASS} || 'Win32',
-		      undef, {not_lazy=>0, autocommit=>0, lifetime=>&Apache::Session::LIFETIME} ;
-	warn "[$$]SES:  Embperl Session management enabled (0.17)\n" ;
+	##$SessionMgnt = 1 ;
+	##tie %udat, 'Apache::Session', $ENV{EMBPERL_SESSION_CLASS} || 'Win32',
+	##	      undef, {not_lazy=>0, autocommit=>0, lifetime=>&Apache::Session::LIFETIME} ;
+	##tie %mdat, 'Apache::Session', $ENV{EMBPERL_SESSION_CLASS} || 'Win32',
+	##	      undef, {not_lazy=>0, autocommit=>0, lifetime=>&Apache::Session::LIFETIME} ;
+	warn "[$$]SES:  Apache::Session 0.17 not supported by Embperl Session management anymore\n" ;
+	$SessionMgnt = 0 ;
         }
     }
 
-#######################################################################################
-
-#no strict ;
-
-sub _eval_ ($)
-    {
-    my $result = eval "package $evalpackage ; $_[0] " ;
-    if ($@ ne '')
-        { logevalerr ($@) ; }
-    return $result ;
-    }
-
-#use strict ;
-
-#######################################################################################
-
-sub _evalsub_ ($)
-    {
-    my $result = eval "package $evalpackage ; sub { $_[0] } " ;
-    if ($@ ne '')
-        { logevalerr ($@) ; }
-    return $result ;
-    }
 
 #######################################################################################
 
@@ -555,11 +559,13 @@ sub SendLogFile ($$$)
 sub CheckFile
 
     {
-    my ($filename, $req_rec, $AllowZeroFilesize, $allow) = @_ ;
+    my ($filename, $req_rec, $AllowZeroFilesize, $allow, $pathref, $debug) = @_ ;
+
+    my $path = $$pathref ;
 
     if (-d $filename)
         {
-	#logerror (rcIsDir, $filename, $req_rec);
+	#logerror (rcIsDir, $filename);
 	return &DECLINED ; # let Apache handle directories
 	}                 
 
@@ -569,24 +575,60 @@ sub CheckFile
 	return &FORBIDDEN ;
  	}
 	
+    if (defined ($req_rec) && !($req_rec->allow_options & &OPT_EXECCGI))
+        {
+	logerror (rcExecCGIMissing, $filename);
+	return &FORBIDDEN ;
+ 	}
+
+    if ($path && !($filename =~ m{^(/|\\|\w:/|\w:\\|\./|\.\\)}))
+        {
+        my $skip = 0 ;
+        if ($filename =~ m{^(\.\./|\.\.\\)+?.*?(/|\\)*?(.*?)$}) 
+            {
+            $filename = $3 ;
+            $skip = length ($1) / 3 ;
+            }
+        my @path = split /$pathsplit/o, $path ;
+        shift @path while (!$path[0]) ;
+        shift @path while ($skip--) ;
+        my $fn = '' ;
+        print LOG "[$$]Embperl path search Path: " . join (';',@path) . " Filename: $filename\n" if ($debug);
+        $$pathref = join (';', @path) ;
+
+        foreach (@path)
+            {
+            next if (!$_) ;
+            $fn = "$_/$filename" ;
+            print LOG "[$$]Embperl path search Check: $fn\n" if ($debug);
+            if (-r $fn && (-s _ || $AllowZeroFilesize))
+                {
+                if (defined ($allow) && !($fn =~ /$allow/))
+                    {
+	            logerror (rcNotAllowed, $fn, $req_rec);
+	            return &FORBIDDEN ;
+ 	            }
+                $_[0] = $fn ;
+                return ok ;
+                }
+            }
+
+        -r $filename ;
+        }            
+
     unless (-r _ && (-s _ || $AllowZeroFilesize))
         {
-	logerror (rcNotFound, $filename, $req_rec);
+        logerror (rcNotFound, $filename);
 	return &NOT_FOUND ;
         }
 
-    if (defined ($req_rec) && !($req_rec->allow_options & &OPT_EXECCGI))
-        {
-	logerror (rcExecCGIMissing, $filename, $req_rec);
-	return &FORBIDDEN ;
- 	}
-	
     return ok ;
     }
 
+
 ##########################################################################################
 
-sub ScanEnvironement
+sub ScanEnvironment
 
     {
     my ($req, $req_rec) = @_ ; 
@@ -608,9 +650,12 @@ sub ScanEnvironement
     $$req{'input_func'}  = $ENV{EMBPERL_INPUT_FUNC}  if (exists ($ENV{EMBPERL_INPUT_FUNC})) ;
     $$req{'output_func'} = $ENV{EMBPERL_OUTPUT_FUNC} if (exists ($ENV{EMBPERL_OUTPUT_FUNC})) ;
     $$req{'allow'}       = $ENV{EMBPERL_ALLOW}       if (exists ($ENV{EMBPERL_ALLOW})) ;
+    $$req{'filesmatch'}  = $ENV{EMBPERL_FILESMATCH}  if (exists ($ENV{EMBPERL_FILESMATCH})) ;
+    $$req{'decline'}     = $ENV{EMBPERL_DECLINE}     if (exists ($ENV{EMBPERL_DECLINE})) ;
     $$req{'debug'}       = $ENV{EMBPERL_DEBUG}   || 0 ;
     $$req{'options'}     = $ENV{EMBPERL_OPTIONS} || 0 ;
     $$req{'log'}         = $ENV{EMBPERL_LOG}     || $DefaultLog ;
+    $$req{'path'}        = $ENV{EMBPERL_PATH}    || '' ;
 
     if (defined($ENV{EMBPERL_ESCMODE}))
         { $$req{'escmode'}    = $ENV{EMBPERL_ESCMODE} }
@@ -624,6 +669,9 @@ sub ScanEnvironement
     }
 
 
+*ScanEnvironement = \&ScanEnvironment ; # for backward compatibility (was typo)
+
+
 
 #######################################################################################
 
@@ -676,6 +724,13 @@ sub Execute
         $opcodemask = $cp -> mask ;
         }
 
+    if (exists ($req -> {'cookie_expires'}) && ($$req{'cookie_expires'} =~ /^\+|\-/))
+        {
+        require CGI ;
+
+        $req -> {'cookie_expires'} = CGI::expires($req -> {'cookie_expires'}, 'cookie') ;
+        }
+
     my $conf = SetupConfData ($req, $opcodemask) ;
 
     
@@ -732,6 +787,23 @@ use strict ;
 
     my $Inputfile    = $$req{'inputfile'} || '?' ;
     my $Sub          = $$req{'sub'} || '' ;
+    my $lastreq      = CurrReq () ;
+    
+    if ($lastreq)
+        {
+        if ($Inputfile eq '*') 
+            {
+            $Inputfile = $lastreq -> ReqFilename ;
+            }
+        elsif ($Inputfile eq '../*') 
+            {
+            my $fn = $lastreq -> ReqFilename ;
+            $fn =~ m{^.*/(.*?)$} ;
+            $Inputfile = "../$1" ;
+            }
+        $$req{'path'} ||= $lastreq -> Path  ;
+        $$req{'debug'} ||= $lastreq -> Debug  ;
+        }
     
     if (defined ($In))
         {
@@ -740,7 +812,10 @@ use strict ;
         }
    elsif (!$Sub || $Inputfile ne '?')
         {
-        if ($rc = CheckFile ($Inputfile, $req_rec, (($$req{options} || 0) & optAllowZeroFilesize), $$req{'allow'})) 
+        #my ($k, $v) ;
+        #while (($k, $v) = each (%$req))
+        #    { warn "$k = $v" ; }
+        if ($rc = CheckFile ($Inputfile, $req_rec, (($$req{options} || 0) & optAllowZeroFilesize), $$req{'allow'}, \$req -> {path}, (($$req{debug} || 0) & dbgObjectSearch))) 
             {
 	    FreeConfData ($conf) ;
             return $rc ;
@@ -754,11 +829,16 @@ use strict ;
         $mtime = 0 ;
 	}
 
+
     my $ar  ;
     $ar = Apache->request if (defined ($req_rec)) ; # workaround that Apache::Request has another C Interface, than Apache
     my $r = SetupRequest ($ar, $Inputfile, $mtime, $filesize, ($$req{firstline} || 1), $Outputfile, $conf,
                           &epIOMod_Perl, $In, $Out, $Sub, exists ($$req{import})?scalar(caller ($$req{import} > 0?$$req{import} - 1:0)):'',$SessionMgnt) ;
     
+    bless $r, $$req{'bless'} if (exists ($$req{'bless'})) ;
+
+    $r -> Path ($req->{path}) if ($req->{path}) ;
+
     my $package = $r -> CurrPackage ;
     $evalpackage = $package ;   
     my $exports ;
@@ -780,6 +860,7 @@ use strict ;
 	    @ffld = keys %fdat if (!defined ($$req{'ffld'})) ;
 	    }
 	elsif (!($optDisableFormData) &&
+	       !($r -> SubReq) &&
 	       defined($ENV{'CONTENT_TYPE'}) &&
 	       $ENV{'CONTENT_TYPE'}=~m|^multipart/form-data|)
 	    { # just let CGI.pm read the multipart form data, see cgi docu
@@ -820,43 +901,17 @@ use strict ;
 
 	    }
 
-    no strict ;
-	# pass parameters via @param
-	*{"$package\:\:param"}   = $$req{'param'} if (exists $$req{'param'}) ;
-    use strict ;
-    
-	my $udat ;
-	my $mdat ;
+	my $saved_param = undef;
+	if ( ref $$req{'param'} eq 'ARRAY') {
+	    no strict 'refs';
+	    # pass parameters via @param
+	    $saved_param = \@{"$package\:\:param"} 
+		if defined @{"$package\:\:param"};
+	    *{"$package\:\:param"}   = $$req{'param'};
+	}
 
-	if ($SessionMgnt && !$r -> SubReq)
-	    {
-	    $udat = tied(%udat) ;
-	    $mdat = tied(%mdat) ;
-	    my $sessid ;
-	    my $cookie_name = $r -> CookieName ;
-
-	    if (defined ($ENV{HTTP_COOKIE}) && ($ENV{HTTP_COOKIE} =~ /$cookie_name=(.*?)(\;|\s|$)/))
-		{
-		$sessid = $1 ;
-		print LOG "[$$]SES:  Received session cookie $1\n" if ($dbgSession) ;
-                $r -> SessionMgnt (0) ; # do not resend cookie
-		}
-
-	    if ($SessionMgnt == 1)
-		{
-		$udat -> {ID} = $sessid ;
-		$udat -> {DIRTY} = 0 ;
-
-		$mdat -> {ID} = substr(MD5 -> hexhash ($Inputfile), 0, &Apache::Session::ID_LENGTH );
-		$mdat -> {DIRTY} = 0 ;
-		}
-	    else
-		{
-		$udat -> setid ($sessid) ;
-		$mdat -> setid (substr(MD5 -> hexhash ($Inputfile), 0, 16));
-		}
-	    }
 
+        $r -> SetupSession ($req_rec, $Inputfile) ;
 
 	    {
 	    local $SIG{__WARN__} = \&Warn ;
@@ -865,7 +920,8 @@ use strict ;
 	    my $saver = $r ;
         
 
-	    $rc = CleanCallExecuteReq ($r, $$req{'param'}) ;
+	    $@ = undef ;
+            $rc = CleanCallExecuteReq ($r, $$req{'param'}) ;
 	    #$rc = $r -> ExecuteReq ($$req{'param'}) ;
         
 	    $r = $saver ;
@@ -893,43 +949,14 @@ use strict ;
 	    }
 
 
-	no strict ;
-	undef *{"$package\:\:param"} ;
-	use strict ;
 
-    
-	if ($SessionMgnt && !$r -> SubReq)
-	    {
-	    if ($SessionMgnt == 1)
-		{
-		if ($udat->{'DIRTY'})
-		    {
-		    print LOG "[$$]SES:  Store session data of \%udat id=$udat->{ID}\n" if ($dbgSession) ;
-		    $udat->{'DATA'}->store ;
-		    }
-		else
-		    {
-		    print LOG "[$$]SES:  session data not dirty, do not store \%udat\n" if ($dbgSession) ;
-		    }
+	if ( defined $saved_param ) {
+	    no strict 'refs';
+	    *{"$package\:\:param"} = $saved_param;
+	}
 
-		$udat->{'DATA'} = undef ;
-		$udat -> {ID}	= undef ;
-		if ($mdat->{'DIRTY'})
-		    {
-		    print LOG "[$$]SES:  Store session data of \%mdat id=$mdat->{ID}\n" if ($dbgSession) ;
-		    $mdat->{'DATA'}->store ;
-		    }
-
-		$mdat->{'DATA'} = undef ;
-		$mdat -> {ID}	= undef ;
-		}
-	    else
-		{
-		$udat -> cleanup ;
-		$mdat -> cleanup ;
-		}
-	    }
 
+        $r -> CleanupSession ;
 
         $r -> Export ($exports, caller ($$req{import} - 1)) if ($$req{import} && ($exports = $r -> ExportHash)) ;
 
@@ -1007,7 +1034,7 @@ sub run (\@)
     my $ioType ;
     my %req ;
 
-    ScanEnvironement (\%req) ;
+    ScanEnvironment (\%req) ;
     
 
     if (defined ($$args[0]) && $$args[0] eq 'dbgbreak') 
@@ -1101,7 +1128,7 @@ sub runcgi ()
     my $ioType ;
     my %req ;
 
-    ScanEnvironement (\%req) ;
+    ScanEnvironment (\%req) ;
     
     $req{'inputfile'} = $ENV{PATH_TRANSLATED} ;
     $ioType = epIOCGI ;
@@ -1138,10 +1165,11 @@ sub handler
 
     my %req ;
 
-    ScanEnvironement (\%req, $req_rec) ;
+    ScanEnvironment (\%req, $req_rec) ;
     
     $req{'uri'}       = $req_rec -> Apache::uri ;
 
+    #warn "1 uri = $req{'uri'}\n" ;
     if (exists $ENV{EMBPERL_FILESMATCH} && 
                          !($req{'uri'} =~ m{$ENV{EMBPERL_FILESMATCH}})) 
         {
@@ -1153,14 +1181,22 @@ sub handler
 
     $req{'inputfile'} = $ENV{PATH_TRANSLATED} = $req_rec -> filename ;
 
-    #print LOG "i = $req{'inputfile'}\n" ;
+    #warn "ok inputfile = $req{'inputfile'}\n" ;
 
-    $req{'cleanup'} = -1 if (($req{'options'} & optDisableVarCleanup)) ;
+    $req{'cleanup'} = -1 if (($req{'options'} & optDisableVarCleanup));
     $req{'options'} |= optSendHttpHeader ;
     $req{'req_rec'} = $req_rec ;
+    my @errors ;
+    $req{'errors'} = \@errors ;
+    $req_rec -> pnotes ('EMBPERL_ERRORS', \@errors) if (defined (&Apache::pnotes)) ;      
 
     my $rc = Execute (\%req) ;
 
+    #print LOG "errors1=@errors\n" ;
+
+    my $e = $req_rec -> pnotes ('EMBPERL_ERRORS') ;
+    #print LOG "errors2=@$e\n" ;
+
     #log_svs ("handler exit") ;
     return $rc ;
     }
@@ -1183,7 +1219,9 @@ sub cleanup
     my $packfile ;
     my %addcleanup ;
     my $varfile ;
-
+    my %revinc = map { ($_ => 1) } values (%INC) if ($multiplicity) ;
+    my ($k, $v) ;
+    
     $seen{''}      = 1 ;
     $seen{'dbgShowCleanup'} = 1 ;
     foreach $package (@cleanups)
@@ -1193,8 +1231,14 @@ sub cleanup
 
 	$seen{$package} = 1 ;
         
+        #print LOG "GVFile $package\::__ANON__\n" ;
 	$packfile = GVFile (*{"$package\::__ANON__"}) ;
-        $packfile = '-> No Perl in Source <-' if ($packfile eq ('_<' . __FILE__)) ;
+	if ($multiplicity && !$revinc{$packfile})
+            {
+            print LOG "$packfile -> -- eval --\n" ;
+            $packfile = "-- eval --" ;
+            }
+        $packfile = '-> No Perl in Source <-' if ($packfile eq ('_<' . __FILE__) || $packfile eq __FILE__) ;
 	$addcleanup = \%{"$package\:\:CLEANUP"} ;
 	$addcleanup -> {'CLEANUP'} = 0 ;
 	if ($Debugflags & dbgShowCleanup)
@@ -1208,27 +1252,35 @@ sub cleanup
 	    #eval "\&$1\:\:CLEANUP;" ;
 	    eval "\&$package\:\:CLEANUP;" ;
 	    print LOG "[$$]CUP:  Call \&$package\:\:CLEANUP;\n" if ($Debugflags & dbgShowCleanup);
-	    embperl_logevalerr ($@) if ($@) ;
+	    logevalerr ($@) if ($@) ;
 	    }
 
 
         if ($Debugflags & dbgShowCleanup)
             {
 	    my @vars = sort keys %{*{"$package\::"}} ;
+            my $cleanfile = \%{"$package\:\:CLEANUPFILE"} ;
 	    foreach $key (@vars)
 		{
 		$val =  ${*{"$package\::"}}{$key} ;
 		local(*ENTRY) = $val;
 		#print LOG "$key = " . GVFile (*ENTRY) . "\n" ;
 		$varfile = GVFile (*ENTRY) ;
-	        $glob = $package.'::'.$key ;
+	        if ($multiplicity && !$revinc{$varfile})
+                    {
+                    print LOG "$varfile -> -- eval --\n" ;
+                    $varfile = "-- eval --" ;
+                    }
+                
+                $glob = $package.'::'.$key ;
 		if (defined (*ENTRY{SCALAR}) && defined (${$glob}) && ref (${$glob}) eq 'DBIx::Recordset')
 		    {
 		    print LOG "[$$]CUP:  Recordset $key\n" ;
 		    eval { DBIx::Recordset::Undef ($glob) ; } ;
 		    print LOG "[$$]CUP:  Error: $@\n" if ($@) ;
 		    } 
-		elsif (($packfile eq $varfile || $addcleanup -> {$key}) && 
+		elsif (($packfile eq $varfile || $addcleanup -> {$key} ||
+                        $cleanfile->{$varfile}) &&  
 		     (!($key =~ /\:\:$/) && !(defined ($addcleanup -> {$key}) && $addcleanup -> {$key} == 0)))
 		    { # Only cleanup vars which are defined in the sourcefile
 		      # ignore all imported vars, unless they are in the CLEANUP hash which is set by VARS
@@ -1250,7 +1302,8 @@ sub cleanup
 			my $i = 0 ;
 			my $k ;
 			my $v ;
-			while (($k, $v) = each (%{$glob}))
+                        eval { # ignore errors here (for ActiveState Perl)
+                        while (($k, $v) = each (%{$glob}))
 			    {
 			    if ($i++ > 5) 
 				{
@@ -1258,7 +1311,7 @@ sub cleanup
 				last 
 				}
 			    print LOG "$k => $v, "
-			    }
+			    } } ;
 			print LOG ")\n" ;
 			eval { untie %{$glob} ; } ;
 			print LOG "[$$]CUP:  Error: $@\n" if ($@) ;
@@ -1293,6 +1346,7 @@ sub cleanup
             }
         else
             {
+            my $cleanfile = \%{"$package\:\:CLEANUPFILE"} ;
             while (($key,$val) = each(%{*{"$package\::"}}))
                 {
 	        local(*ENTRY) = $val;
@@ -1302,34 +1356,45 @@ sub cleanup
 		    eval { DBIx::Recordset::Undef ($glob) ; } ;
 		    print LOG "[$$]CUP:  Error: $@\n" if ($@) ;
 		    } 
-		elsif (($packfile eq GVFile (*ENTRY) || $addcleanup -> {$key}) && 
-		     (!($key =~ /\:\:$/) && !(defined ($addcleanup -> {$key}) && $addcleanup -> {$key} == 0)))
-		    { # Only cleanup vars which are defined in the sourcefile
-		      # ignore all imported vars, unless they are in the CLEANUP hash which is set by VARS
-                    if (defined (*ENTRY{SCALAR}) && defined (${$glob})) 
-			{
-			eval { undef ${$glob} ; } ;
-			print LOG "[$$]CUP:  Error while cleanup \$$glob: $@\n" if ($@) ;
-			}
-		    if (defined (*ENTRY{IO})) 
-			{
-			eval { close *{"$package\:\:$key"} ; } ;
-			print LOG "[$$]CUP:  Error while closing $glob: $@\n" if ($@) ;
-			}
-		    if (defined (*ENTRY{HASH})) 
-			{
-			eval { untie %{$glob} ; } ;
-			print LOG "[$$]CUP:  Error while cleanup \%$glob: $@\n" if ($@) ;
-			eval { undef %{$glob} ; } ;
-			print LOG "[$$]CUP:  Error while cleanup \%$glob: $@\n" if ($@) ;
-			}
-		    if (defined (*ENTRY{ARRAY})) 
-			{
-			eval { untie @{$glob} ; } ;
-			print LOG "[$$]CUP:  Error while cleanup \@$glob: $@\n" if ($@) ;
-			eval { undef @{$glob} ; } ;
-			print LOG "[$$]CUP:  Error while cleanup \@$glob: $@\n" if ($@) ;
-			}
+		else
+                    {
+		    $varfile = GVFile (*ENTRY) ;
+	            if ($multiplicity && !$revinc{$varfile})
+                        {
+                        print LOG "$varfile -> -- eval --\n" ;
+                        $varfile = "-- eval --" ;
+                        }
+
+                    if (($packfile eq $varfile || $addcleanup -> {$key} || 
+                        $cleanfile->{$varfile}) &&  
+		         (!($key =~ /\:\:$/) && !(defined ($addcleanup -> {$key}) && $addcleanup -> {$key} == 0)))
+		        { # Only cleanup vars which are defined in the sourcefile
+		          # ignore all imported vars, unless they are in the CLEANUP hash which is set by VARS
+                        if (defined (*ENTRY{SCALAR}) && defined (${$glob})) 
+			    {
+			    eval { undef ${$glob} ; } ;
+			    print LOG "[$$]CUP:  Error while cleanup \$$glob: $@\n" if ($@) ;
+			    }
+		        if (defined (*ENTRY{IO})) 
+			    {
+			    eval { close *{"$package\:\:$key"} ; } ;
+			    print LOG "[$$]CUP:  Error while closing $glob: $@\n" if ($@) ;
+			    }
+		        if (defined (*ENTRY{HASH})) 
+			    {
+			    eval { untie %{$glob} ; } ;
+			    print LOG "[$$]CUP:  Error while cleanup \%$glob: $@\n" if ($@) ;
+			    eval { undef %{$glob} ; } ;
+			    print LOG "[$$]CUP:  Error while cleanup \%$glob: $@\n" if ($@) ;
+			    }
+		        if (defined (*ENTRY{ARRAY})) 
+			    {
+			    eval { untie @{$glob} ; } ;
+			    print LOG "[$$]CUP:  Error while cleanup \@$glob: $@\n" if ($@) ;
+			    eval { undef @{$glob} ; } ;
+			    print LOG "[$$]CUP:  Error while cleanup \@$glob: $@\n" if ($@) ;
+			    }
+                        }
 		    }
 		}
             }
@@ -1386,8 +1451,12 @@ sub MailFormTo
     #die "require Net::SMTP failed: $@" if ($@); 
     require Net::SMTP ;
 
-    $smtp = Net::SMTP->new($ENV{'EMBPERL_MAILHOST'} || 'localhost', Debug => $ENV{'EMBPERL_MAILDEBUG'}) or die "Cannot connect to mailhost" ;
-    { $smtp->mail($ENV{'EMBPERL_MAILFROM'} || "WWW-Server\@$ENV{SERVER_NAME}");}
+    $smtp = Net::SMTP->new($ENV{'EMBPERL_MAILHOST'} || 'localhost', 
+                           Debug => $ENV{'EMBPERL_MAILDEBUG'} || 0,
+                           $ENV{'EMBPERL_MAILHELO'}?(Hello => $ENV{'EMBPERL_MAILDEBUG'}):()) 
+             or die "Cannot connect to mailhost" ;
+    
+    $smtp->mail($ENV{'EMBPERL_MAILFROM'} || "WWW-Server\@$ENV{SERVER_NAME}");
     $smtp->to($to);
     $ok = $smtp->data();
     $ok = $smtp->datasend("Reply-To: $ret\n") if ($ok && $ret) ;
@@ -1529,6 +1598,187 @@ if (defined ($ENV{MOD_PERL}))
     }
 
 
+#######################################################################################
+
+sub SetupSession
+
+    {
+    my $r ;
+    $r = shift if (!(ref ($_[0]) =~ /^Apache/)) ;
+    my ($req_rec, $Inputfile) = @_ ;
+
+    if ($HTML::Embperl::SessionMgnt && (!defined ($r) || !$r -> SubReq))
+	{
+	my $udat = tied(%HTML::Embperl::udat) ;
+	my $mdat = tied(%HTML::Embperl::mdat) ;
+	my $sessid ;
+	my $cookie_name = $r?$r -> CookieName:$ENV{EMBPERL_COOKIE_NAME} || 'EMBPERL_UID' ;
+        my $cookie_val  = $ENV{HTTP_COOKIE} || ($req_rec?$req_rec->header_in('Cookie'):undef) ;
+
+	if (defined ($cookie_val) && ($cookie_val =~ /$cookie_name=(.*?)(\;|\s|$)/))
+	    {
+	    $sessid = $1 ;
+	    print HTML::Embperl::LOG "[$$]SES:  Received session cookie $1\n" if ($HTML::Embperl::dbgSession) ;
+            }
+
+	if ($HTML::Embperl::SessionMgnt == 1)
+	    {
+            if (!$udat -> {ID})
+                {
+	        $udat -> {ID} = $sessid ;
+	        $udat -> {DIRTY} = 0 ;
+                }
+
+            if ($Inputfile && !$mdat -> {ID})
+                {
+	        $mdat -> {ID} = substr(MD5 -> hexhash ($Inputfile), 0, &Apache::Session::ID_LENGTH );
+	        $mdat -> {DIRTY} = 0 ;
+                }
+	    }
+	else
+	    {
+	    $udat -> setid ($sessid) if (!$udat -> getid) ;
+	    $mdat -> setid (substr(MD5 -> hexhash ($Inputfile), 0, $mdat -> {args} -> {IDLength} || $HTML::Embperl::DefaultIDLength)) if ($Inputfile && !$mdat -> getid)
+	    }
+	}
+    else
+        {
+        return undef ; # No session Management
+        }
+
+    return wantarray?(\%HTML::Embperl::udat, \%HTML::Embperl::mdat):\%HTML::Embperl::udat ;
+    }
+
+#######################################################################################
+
+sub GetSession
+
+    {
+    if ($HTML::Embperl::SessionMgnt)
+	{
+	my $udat = tied(%HTML::Embperl::udat) ;
+
+	if ($HTML::Embperl::SessionMgnt == 1)
+	    {
+            return wantarray?(\%HTML::Embperl::udat, \%HTML::Embperl::mdat):\%HTML::Embperl::udat ;
+	    }
+	else
+	    {
+	    return wantarray?(\%HTML::Embperl::udat, \%HTML::Embperl::mdat):\%HTML::Embperl::udat  ;
+	    }
+	}
+    else
+        {
+        return undef ; # No session Management
+        }
+    }
+
+#######################################################################################
+
+sub DeleteSession
+
+    {
+    my $r = shift || HTML::Embperl::CurrReq () ;
+    my $disabledelete = shift ;
+
+    my $udat = tied (%HTML::Embperl::udat) ;
+    if (!$disabledelete)  # Delete session data
+        {
+        $udat -> delete  ;
+        }
+    else
+        {
+        $udat-> {data} = {} ; # for make test only
+        $udat->{initial_session_id} = "!DELETE" ;
+        }
+    $udat->{status} = 0;
+    }
+
+
+#######################################################################################
+
+sub RefreshSession
+
+    {
+    my $r = shift || HTML::Embperl::CurrReq () ;
+
+    $r -> SessionMgnt ($HTML::Embperl::SessionMgnt | 4) ; # resend cookie 
+    }
+
+#######################################################################################
+
+sub CleanupSession
+
+    {
+    my $r = shift ;
+    $r = HTML::Embperl::CurrReq () if (!(ref ($r) =~ /^HTML::Embperl/));
+
+    if ($HTML::Embperl::SessionMgnt && (!defined ($r) || !$r -> SubReq))
+	{
+	my $udat = tied(%HTML::Embperl::udat) ;
+	my $mdat = tied(%HTML::Embperl::mdat) ;
+
+	if ($HTML::Embperl::SessionMgnt & 1)
+	    {
+	    if ($udat->{'DIRTY'})
+		{
+		print HTML::Embperl::LOG "[$$]SES:  Store session data of \%HTML::Embperl::udat id=$udat->{ID}\n" if ($HTML::Embperl::dbgSession) ;
+		$udat->{'DATA'}->store ;
+		}
+	    else
+		{
+		print HTML::Embperl::LOG "[$$]SES:  session data not dirty, do not store \%HTML::Embperl::udat\n" if ($HTML::Embperl::dbgSession) ;
+		}
+
+	    $udat->{'DATA'} = undef ;
+	    $udat -> {ID}	= undef ;
+	    if ($mdat->{'DIRTY'})
+		{
+		print HTML::Embperl::LOG "[$$]SES:  Store session data of \%HTML::Embperl::mdat id=$mdat->{ID}\n" if ($HTML::Embperl::dbgSession) ;
+		$mdat->{'DATA'}->store ;
+		}
+
+	    $mdat->{'DATA'} = undef ;
+	    $mdat -> {ID}	= undef ;
+	    }
+	else
+	    {
+	    $udat -> cleanup ;
+	    $mdat -> cleanup ;
+	    }
+	}
+    }
+
+
+#######################################################################################
+
+sub SetSessionCookie
+
+    {
+    my $r = shift ;
+    $r = undef if (!(ref ($r) =~ /^HTML::Embperl/));
+
+    if ($HTML::Embperl::SessionMgnt)
+        {
+        my $udat   = tied (%HTML::Embperl::udat) ;
+        my $id     = $udat -> getid ;
+        my $initialid     = $udat -> getinitialid ;
+        
+        my $name   = $ENV{EMBPERL_COOKIE_NAME} || 'EMBPERL_UID' ;
+        my $domain = "; domain=$ENV{EMBPERL_COOKIE_DOMAIN}" if (exists ($ENV{EMBPERL_COOKIE_DOMAIN})) ;
+        my $path   = "; path=$ENV{EMBPERL_COOKIE_PATH}" if (exists ($ENV{EMBPERL_COOKIE_PATH})) ;
+        my $expires = "; expires=$ENV{EMBPERL_COOKIE_EXPIRES}" if (exists ($ENV{EMBPERL_COOKIE_EXPIRES})) ;
+        $expires = "; expires=Thu, 1-Jan-1970 00:00:01 GMT" if ($id && !$initialid) ;
+                        
+        if ($id || $initialid)
+            {    
+            Apache -> request -> header_out ("Set-Cookie" => "$name=$id$domain$path$expires") ;
+            }
+        }
+    }
+
+
+
 
 #######################################################################################
 
@@ -1559,7 +1809,7 @@ sub CreateAliases
         *{"$package\:\:tabmode"} = \$HTML::Embperl::tabmode ;
         *{"$package\:\:escmode"} = \$HTML::Embperl::escmode ;
         *{"$package\:\:http_headers_out"} = \%HTML::Embperl::http_headers_out ;
-    	*{"$package\:\:req_rec"} = \$HTML::Embperl::req_rec if defined ($HTML::Embperl::req_rec) ;
+    	*{"$package\:\:req_rec"} = \$HTML::Embperl::req_rec ; 
     	if (defined (&Apache::exit))
             {
             *{"$package\:\:exit"}    = \&Apache::exit 
@@ -1601,7 +1851,6 @@ sub CreateAliases
 
 
         *{"$package\:\:dbgAllCmds"}               = \$HTML::Embperl::dbgAllCmds           ;
-        *{"$package\:\:dbgCacheDisable"}          = \$HTML::Embperl::dbgCacheDisable      ;
         *{"$package\:\:dbgCmd"}                   = \$HTML::Embperl::dbgCmd               ;
         *{"$package\:\:dbgDefEval"}               = \$HTML::Embperl::dbgDefEval           ;
         *{"$package\:\:dbgEarlyHttpHeader"}       = \$HTML::Embperl::dbgEarlyHttpHeader   ;
@@ -1645,12 +1894,9 @@ sub Export
     print HTML::Embperl::LOG  "[$$]IMP:  Create Imports for $caller from $package ($exports)\n" ;
     no strict ;
 
-    my $v ;
-    my $k ;
-
-    while (($k, $v) = each (%$exports))
+    foreach $k (keys %$exports)
 	{
-        *{"$caller\:\:$k"}    = $v ; #\&{"$package\:\:$k"} ;
+        *{"$caller\:\:$k"}    = $exports -> {$k} ; #\&{"$package\:\:$k"} ;
         print HTML::Embperl::LOG  "[$$]IMP:  Created Import for $package\:\:$k -> $caller\n" ;
         }
 
@@ -1775,7 +2021,7 @@ sub MailErrorsTo ()
         $ok and $ok = $smtp->datasend("$k\t= \"$v\" \n" );
         }
     $ok and $ok = $smtp->datasend("-------------\r\n");
-    $ok and $ok = $smtp->datasend("Environement:\r\n");
+    $ok and $ok = $smtp->datasend("Environment:\r\n");
     $ok and $ok = $smtp->datasend("-------------\r\n");
 
     my $env = $self -> EnvHash() ;
