@@ -67,7 +67,9 @@ enum tRc
     rcCGIError,
     rcUnclosedHtml,
     rcUnclosedCmd,
-    rcNotAllowed
+    rcNotAllowed,
+    rcNotHashRef,
+    rcTagMismatch
     
     } ;
 
@@ -101,6 +103,10 @@ enum dbg
     dbgProfile          = 0x100000,
     dbgSession          = 0x200000,
     dbgImport		= 0x400000,
+    dbgBuildToken       = 0x800000,
+    dbgParse            = 0x1000000,
+    dbgObjectSearch     = 0x2000000,
+    dbgCache            = 0x4000000,
     
     dbgAll  = -1
     } ;
@@ -133,7 +139,8 @@ enum opt
     optKeepSrcInMemory         = 0x80000,
     optKeepSpaces	       = 0x100000,
     optOpenLogEarly            = 0x200000,
-    optNoUncloseWarn	       = 0x400000
+    optNoUncloseWarn	       = 0x400000,
+    optDisableSelectScan       = 0x800000
     } ;
 
 /*
@@ -176,7 +183,8 @@ enum tEscMode
     escNone = 0,
     escHtml = 1,
     escUrl  = 2,
-    escStd  = 3
+    escStd  = 3,
+    escEscape  = 4
     } ;
 
 
