@@ -61,6 +61,78 @@ extern "C" {
 #undef FALSE
 #endif
 
+#ifdef WIN32
+
+#ifdef uid_t
+#define apache_uid_t uid_t
+#undef uid_t
+#endif
+#define uid_t apache_uid_t
+
+#ifdef gid_t
+#define apache_gid_t gid_t
+#undef gid_t
+#endif
+#define gid_t apache_gid_t
+
+#ifdef mode_t
+#define apache_mode_t mode_t
+#undef mode_t
+#endif
+#define mode_t apache_mode_t
+
+#ifdef stat
+#define apache_stat stat
+#undef stat
+#endif
+
+#ifdef lstat
+#define apache_lstat lstat
+#undef lstat
+#endif
+
+#ifdef sleep
+#define apache_sleep sleep
+#undef sleep
+#endif
+
+#ifndef PERL_VERSION
+#include <patchlevel.h>
+#define PERL_VERSION PATCHLEVEL
+#define PERL_SUBVERSION SUBVERSION
+#endif
+
+#if PERL_VERSION >= 6
+
+#ifdef opendir
+#define apache_opendir opendir
+#undef opendir
+#endif
+
+#ifdef readdir
+#define apache_readdir readdir
+#undef readdir
+#endif
+
+#ifdef closedir
+#define apache_closedir closedir
+#undef closedir
+#endif
+
+#ifdef crypt
+#define apache_crypt crypt
+#undef crypt
+#endif
+
+#ifdef errno
+#define apache_errno errno
+#undef errno
+#endif
+
+#endif /* endif PERL_VERSION >= 6 */ 
+
+#endif /* endif WIN32 */
+
 #include <httpd.h>
 #include <http_config.h>
 #include <http_protocol.h>
@@ -72,12 +144,20 @@ extern "C" {
 #endif
 #endif
 
+struct tReq ;
+
+typedef struct tReq req ;
+typedef struct tReq tReq ;
 
 #include "epnames.h"
 
+#ifdef EP2
+#include "epdom.h"
+#endif
 #include "epdat.h"
 #include "embperl.h"
 
+
 #ifdef WIN32
 #define PATH_MAX _MAX_DIR
 #endif
@@ -132,6 +212,11 @@ tFile * SetupFileData   (/*i/o*/ register req * r,
                          /*in*/ int     nFirstLine,
                          /*in*/ tConf * pConf) ;
 
+tFile * GetFileData     (/*in*/  char *  sSourcefile,
+                         /*in*/  char *  sPackage,
+                         /*in*/  double  mtime) ;
+
+			 
 tConf * SetupConfData   (/*in*/ HV *   pReqInfo,
                          /*in*/ SV *   pOpcodeMask) ;
 
@@ -162,6 +247,12 @@ int ProcessSub		(/*i/o*/ register req * r,
 void NewEscMode (/*i/o*/ register req * r,
 			           SV * pSV) ;
 
+
+int AddMagicAV (/*i/o*/ register req * r,
+		/*in*/ char *     sVarName,
+                /*in*/ MGVTBL *   pVirtTab) ;
+
+
 /* ---- from epio.c ----- */
 
 
@@ -293,6 +384,13 @@ int ProcessCmd (/*i/o*/ register req * r,
 			/*in*/ struct tCmd *  pCmd,
                         /*in*/ const char *    sArg) ;
 
+
+SV * SplitFdat     (/*i/o*/ register req * r,
+                           /*in*/  SV ** ppSVfdat,
+                           /*out*/ SV ** ppSVerg,
+                           /*in*/  char * pName,
+                           /*in*/  STRLEN nlen) ;
+
 /* ---- from eputil.c ----- */
 
 
@@ -307,14 +405,17 @@ char * GetHashValueLen (/*in*/  HV *           pHash,
                         /*in*/  int            nMaxLen,
                         /*out*/ char *         sValue) ;
 
-int    GetHashValueInt (/*in*/  HV *           pHash,
+IV    GetHashValueInt (/*in*/  HV *           pHash,
                         /*in*/  const char *   sKey,
-                        /*in*/  int            nDefault) ;
+                        /*in*/  IV            nDefault) ;
 
 char * GetHashValueStr (/*in*/  HV *           pHash,
                         /*in*/  const char *   sKey,
                         /*in*/  char *         sDefault) ;
 
+char * GetHashValueStrDup (/*in*/  HV *           pHash,
+                           /*in*/  const char *   sKey,
+                           /*in*/  char *         sDefault) ;
 
 const char * GetHtmlArg (/*in*/  const char *    pTag,
                          /*in*/  const char *    pArg,
@@ -323,6 +424,12 @@ const char * GetHtmlArg (/*in*/  const char *    pTag,
 void OutputToHtml (/*i/o*/ register req * r,
  		   /*i/o*/ const char *   sData) ;
 
+void OutputEscape (/*i/o*/ register req * r,
+ 		   /*in*/  const char *   sData,
+ 		   /*in*/  int            nDataLen,
+ 		   /*in*/  struct tCharTrans *   pEscTab,
+ 		   /*in*/  char           cEscChar) ;
+
 int TransHtml (/*i/o*/ register req * r,
 		/*i/o*/ char *         sData,
 		/*in*/   int           nLen) ;
@@ -332,6 +439,9 @@ void TransHtmlSV (/*i/o*/ register req * r,
 
 int GetLineNo (/*i/o*/ register req * r) ;
 
+int GetLineNoOf (/*i/o*/ register req * r,
+               /*in*/  char * pPos) ;
+
 #ifndef WIN32
 #define strnicmp strncasecmp
 #endif
@@ -354,9 +464,22 @@ int GetSubTextPos (/*i/o*/ register req * r,
 
 /* ---- from epeval.c ----- */
 
+int CallCV  (/*i/o*/ register req * r,
+		    /*in*/  const char *  sArg,
+                    /*in*/  CV *          pSub,
+                    /*in*/  int           flags,
+                    /*out*/ SV **         pRet) ;
 
-int EvalDirect (/*i/o*/ register req * r,
-			/*in*/  SV * pArg) ;
+int EvalOnly           (/*i/o*/ register req * r,
+			/*in*/  const char *  sArg,
+                        /*in*/  SV **         ppSV,
+                        /*in*/  int           flags,
+  		        /*in*/  const char *  sName) ;
+
+int EvalDirect (/*i/o*/ register req *	r,
+		/*in*/  SV *		pArg,
+                /*in*/  int		numArgs,
+                /*in*/  SV **		pArgs) ;
 
 int EvalNum (/*i/o*/ register req * r,
 	     /*in*/  char *        sArg,
@@ -368,6 +491,13 @@ int Eval (/*i/o*/ register req * r,
           /*in*/  int           nFilepos,
           /*out*/ SV **         pRet) ;
 
+#ifdef EP2
+int EvalStore (/*i/o*/ register req * r,
+	      /*in*/  const char *  sArg,
+	      /*in*/  int           nFilepos,
+	      /*out*/ SV **         pRet) ;
+#endif
+
 int EvalTrans (/*i/o*/ register req * r,
 	       /*in*/  char *   sArg,
                /*in*/  int      nFilepos,
@@ -396,6 +526,32 @@ int EvalSub (/*i/o*/ register req * r,
 
 int EvalMain (/*i/o*/ register req *  r) ;
 
+int EvalConfig (/*i/o*/ register req *  r,
+		/*in*/  SV *            pSV, 
+                /*in*/  int		numArgs,
+                /*in*/  SV **		pArgs,
+		/*out*/ CV **           pCV) ;
+
+
+#ifdef EP2
+int CallStoredCV  (/*i/o*/ register req * r,
+		    /*in*/  const char *  sArg,
+                    /*in*/  CV *          pSub,
+                    /*in*/  int           numArgs,
+                    /*in*/  SV **         pArgs,
+                    /*in*/  int           flags,
+                    /*out*/ SV **         pRet) ;
+#endif
+
+
+
 /* ---- from epdbg.c ----- */
 
 int SetupDebugger (/*i/o*/ register req * r) ;
+
+
+
+#ifdef EP2
+#include "ep2.h"
+#endif
+
