@@ -154,40 +154,39 @@ struct tCharTrans Char2Html [] =
         { '}' ,   ""         },    /* 	&#125;		Right curly brace  */
         { '~' ,   ""         },    /* 	&#126;		Tilde  */
         { '' ,   ""         },    /* 	&#127;		Unused  */
-        { '' ,   ""         },    /* 	&#128;		Unused */
-         
-        { ' ' ,   ""         },    /* &#129;		Unused  */
-        { ' ' ,   ""         },    /* &#130;		Unused  */
-        { ' ' ,   ""         },    /* &#131;		Unused  */
-        { ' ' ,   ""         },    /* &#132;		Unused  */
-        { ' ' ,   ""         },    /* &#133;		Unused  */
-        { ' ' ,   ""         },    /* &#134;		Unused  */
-        { ' ' ,   ""         },    /* &#135;		Unused  */
-        { ' ' ,   ""         },    /* &#136;		Unused  */
-        { ' ' ,   ""         },    /* &#137;		Unused  */
-        { ' ' ,   ""         },    /* &#138;		Horizontal tab  */
-        { ' ' ,   ""         },    /* &#139;		Line feed  */
-        { ' ' ,   ""         },    /* &#140;		Unused  */
-        { ' ' ,   ""         },    /* &#141;		Unused  */
-        { ' ' ,   ""         },    /* &#142;		Carriage Return  */
-        { ' ' ,   ""         },    /* &#143;		Unused  */
-        { ' ' ,   ""         },    /* &#144;		Unused  */
-        { ' ' ,   ""         },    /* &#145;		Unused  */
-        { ' ' ,   ""         },    /* &#146;		Unused  */
-        { ' ' ,   ""         },    /* &#147;		Unused  */
-        { ' ' ,   ""         },    /* &#148;		Unused  */
-        { ' ' ,   ""         },    /* &#149;		Unused  */
-        { ' ' ,   ""         },    /* &#150;		Unused  */
-        { ' ' ,   ""         },    /* &#151;		Unused  */
-        { ' ' ,   ""         },    /* &#152;		Unused  */
-        { ' ' ,   ""         },    /* &#153;		Unused  */
-        { ' ' ,   ""         },    /* &#154;		Unused  */
-        { ' ' ,   ""         },    /* &#155;		Unused  */
-        { ' ' ,   ""         },    /* &#156;		Unused  */
-        { ' ' ,   ""         },    /* &#157;		Unused  */
-        { ' ' ,   ""         },    /* &#158;		Unused  */
-        { ' ' ,   ""         },    /* &#159;		Unused  */
-        { ' ' ,   "&nbsp;"   },    /* 	Non-breaking Space  */
+        { '\x80' ,   "&#128;"         },    /* 	&#128;		Unused */
+        { '\x81' ,   "&#129;"         },    /* &#129;		Unused  */
+        { '\x82' ,   "&#130;"         },    /* &#130;		Unused  */
+        { '\x83' ,   "&#131;"         },    /* &#131;		Unused  */
+        { '\x84' ,   "&#132;"         },    /* &#132;		Unused  */
+        { '\x85' ,   "&#133;"         },    /* &#133;		Unused  */
+        { '\x86' ,   "&#134;"         },    /* &#134;		Unused  */
+        { '\x87' ,   "&#135;"         },    /* &#135;		Unused  */
+        { '\x88' ,   "&#136;"         },    /* &#136;		Unused  */
+        { '\x89' ,   "&#137;"         },    /* &#137;		Unused  */
+        { '\x8a' ,   "&#138;"         },    /* &#138;		Horizontal tab  */
+        { '\x8b' ,   "&#139;"         },    /* &#139;		Line feed  */
+        { '\x8c' ,   "&#140;"         },    /* &#140;		Unused  */
+        { '\x8d' ,   "&#141;"         },    /* &#141;		Unused  */
+        { '\x8e' ,   "&#142;"         },    /* &#142;		Carriage Return  */
+        { '\x8f' ,   "&#143;"         },    /* &#143;		Unused  */
+        { '\x90' ,   "&#144;"         },    /* &#144;		Unused  */
+        { '\x91' ,   "&#145;"         },    /* &#145;		Unused  */
+        { '\x92' ,   "&#146;"         },    /* &#146;		Unused  */
+        { '\x93' ,   "&#147;"         },    /* &#147;		Unused  */
+        { '\x94' ,   "&#148;"         },    /* &#148;		Unused  */
+        { '\x95' ,   "&#149;"         },    /* &#149;		Unused  */
+        { '\x96' ,   "&#150;"         },    /* &#150;		Unused  */
+        { '\x97' ,   "&#151;"         },    /* &#151;		Unused  */
+        { '\x98' ,   "&#152;"         },    /* &#152;		Unused  */
+        { '\x99' ,   "&#153;"         },    /* &#153;		Unused  */
+        { '\x9a' ,   "&#154;"         },    /* &#154;		Unused  */
+        { '\x9b' ,   "&#155;"         },    /* &#155;		Unused  */
+        { '\x9c' ,   "&#156;"         },    /* &#156;		Unused  */
+        { '\x9d' ,   "&#157;"         },    /* &#157;		Unused  */
+        { '\x9e' ,   "&#158;"         },    /* &#158;		Unused  */
+        { '\x9f' ,   "&#159;"         },    /* &#159;		Unused  */
+        { '\xa0' ,   "&nbsp;"   },    /* 	Non-breaking Space  */
         { '' ,   "&iexcl;"  },    /* 	Inverted exclamation    */
         { '' ,   "&cent;"   },    /* 	Cent sign               */
         { '' ,   "&pound;"  },    /* 	Pound sterling  */
@@ -324,14 +323,14 @@ struct tCharTrans Char2Url [] =
         { ' ' ,   "%1D"         },    /* &#29;		Unused  */
         { ' ' ,   "%1E"         },    /* &#30;		Unused  */
         { ' ' ,   "%1F"         },    /* &#31;		Unused  */
-        { ' ' ,   "+"           },    /* 	&#32;		Space  */
+        { ' ' ,   "%20"           },    /* 	&#32;		Space  */
         { '!' ,   ""         },    /* 	&#33;		Exclamation mark  */
         { '"' ,   "%22"   },    /* 	Quotation mark  */
         { '#' ,   ""         },    /* 	&#35;		Number sign  */
         { '$' ,   ""         },    /* 	&#36;		Dollar sign  */
         { '%' ,   "%25"      },    /* 	&#37;		Percent sign  */
         { '&' ,   "%26"    },    /* 	Ampersand  */
-        { '\'' ,  ""         },    /* 	&#39;		Apostrophe  */
+        { '\'' ,  "%27"       },    /* 	&#39;		Apostrophe  */
         { '(' ,   ""         },    /* 	&#40;		Left parenthesis  */
         { ')' ,   ""         },    /* 	&#41;		Right parenthesis  */
         { '*' ,   ""         },    /* 	&#42;		Asterisk  */
@@ -352,9 +351,9 @@ struct tCharTrans Char2Url [] =
         { '9' ,   ""         },    /* 	&#57;		Digit 9  */
         { ':' ,   ""         },    /* 	&#58;		Colon  */
         { ';' ,   ""      },    /* 	&#59;		Semicolon  */
-        { '<' ,   ""      },    /* 	Less than  */
+        { '<' ,   "%3C"      },    /* 	Less than  */
         { '=' ,   "%3D"      },    /* 	&#61;		Equals sign  */
-        { '>' ,   ""      },    /* 	Greater than  */
+        { '>' ,   "%3E"      },    /* 	Greater than  */
         { '?' ,   "%3F"      },    /* 	&#63;		Question mark  */
         { '@' ,   ""      },    /* 	&#64;		Commercial at  */
         { 'A' ,   ""         },    /* 	&#65;		Capital A  */
@@ -555,6 +554,38 @@ struct tCharTrans Char2Url [] =
 struct tCharTrans Html2Char [] = 
  
     {
+        { '\x80' ,   "&#128;"         },    /* &#129;		Unused  */
+        { '\x81' ,   "&#129;"         },    /* &#129;		Unused  */
+        { '\x82' ,   "&#130;"         },    /* &#130;		Unused  */
+        { '\x83' ,   "&#131;"         },    /* &#131;		Unused  */
+        { '\x84' ,   "&#132;"         },    /* &#132;		Unused  */
+        { '\x85' ,   "&#133;"         },    /* &#133;		Unused  */
+        { '\x86' ,   "&#134;"         },    /* &#134;		Unused  */
+        { '\x87' ,   "&#135;"         },    /* &#135;		Unused  */
+        { '\x88' ,   "&#136;"         },    /* &#136;		Unused  */
+        { '\x89' ,   "&#137;"         },    /* &#137;		Unused  */
+        { '\x8a' ,   "&#138;"         },    /* &#138;		Horizontal tab  */
+        { '\x8b' ,   "&#139;"         },    /* &#139;		Line feed  */
+        { '\x8c' ,   "&#140;"         },    /* &#140;		Unused  */
+        { '\x8d' ,   "&#141;"         },    /* &#141;		Unused  */
+        { '\x8e' ,   "&#142;"         },    /* &#142;		Carriage Return  */
+        { '\x8f' ,   "&#143;"         },    /* &#143;		Unused  */
+        { '\x90' ,   "&#144;"         },    /* &#144;		Unused  */
+        { '\x91' ,   "&#145;"         },    /* &#145;		Unused  */
+        { '\x92' ,   "&#146;"         },    /* &#146;		Unused  */
+        { '\x93' ,   "&#147;"         },    /* &#147;		Unused  */
+        { '\x94' ,   "&#148;"         },    /* &#148;		Unused  */
+        { '\x95' ,   "&#149;"         },    /* &#149;		Unused  */
+        { '\x96' ,   "&#150;"         },    /* &#150;		Unused  */
+        { '\x97' ,   "&#151;"         },    /* &#151;		Unused  */
+        { '\x98' ,   "&#152;"         },    /* &#152;		Unused  */
+        { '\x99' ,   "&#153;"         },    /* &#153;		Unused  */
+        { '\x9a' ,   "&#154;"         },    /* &#154;		Unused  */
+        { '\x9b' ,   "&#155;"         },    /* &#155;		Unused  */
+        { '\x9c' ,   "&#156;"         },    /* &#156;		Unused  */
+        { '\x9d' ,   "&#157;"         },    /* &#157;		Unused  */
+        { '\x9e' ,   "&#158;"         },    /* &#158;		Unused  */
+        { '\x9f' ,   "&#159;"         },    /* &#159;		Unused  */
         { '' ,   "&AElig"  },    /* 	Capital AE ligature  */
         { '' ,   "&Aacute" },    /* 	Capital A, acute accent  */
         { '' ,   "&Acirc"  },    /* 	Capital A, circumflex  */
