@@ -94,7 +94,7 @@ struct tCmd CmdTab [] =
         { "/dl",      HtmlEndtable,     0, 1, cmdTable,         0, 0, cnDl     , optDisableTableScan, 1 } ,
         { "/menu",    HtmlEndtable,     0, 1, cmdTable,         0, 0, cnMenu   , optDisableTableScan, 1 } ,
         { "/ol",      HtmlEndtable,     0, 1, cmdTable,         0, 0, cnOl     , optDisableTableScan, 1 } ,
-        { "/select",  HtmlEndtable,     0, 1, cmdTable,         0, 0, cnSelect , optDisableTableScan, 1 } ,
+        { "/select",  HtmlEndtable,     0, 1, cmdTable,         0, 0, cnSelect , optDisableSelectScan, 1 } ,
         { "/table",   HtmlEndtable,     0, 1, cmdTable,         0, 0, cnTable  , optDisableTableScan, 1 } ,
         { "/textarea", HtmlEndtextarea, 0, 1, cmdTextarea,      0, 0, cnNop    , optDisableInputScan, 1 } ,
         { "/tr",      HtmlEndrow,       0, 1, cmdTablerow,      0, 0, cnTr     , optDisableTableScan, 1 } ,
@@ -124,7 +124,7 @@ struct tCmd CmdTab [] =
         { "meta",     HtmlMeta,         0, 0, cmdNorm,          1, 0, cnNop    , optDisableMetaScan , 1 } ,
         { "ol",       HtmlTable,        1, 0, cmdTable,         1, 0, cnOl     , optDisableTableScan, 1 } ,
         { "option",   HtmlOption,       0, 0, cmdNorm,          1, 0, cnNop    , optDisableInputScan, 1 } ,
-        { "select",   HtmlSelect,       1, 0, cmdTable,         1, 0, cnSelect , optDisableTableScan, 1 } ,
+        { "select",   HtmlSelect,       1, 0, cmdTable,         1, 0, cnSelect , optDisableSelectScan, 1 } ,
         { "sub",      CmdSub,           1, 0, cmdSub,           0, 0, cnNop    , 0                  , 0 } ,
         { "table",    HtmlTable,        1, 0, cmdTable,         1, 0, cnTable  , optDisableTableScan, 1 } ,
         { "textarea", HtmlTextarea,     1, 0, cmdTextarea,      1, 1, cnNop    , optDisableInputScan, 1 } ,
@@ -544,7 +544,7 @@ static int CmdUntil (/*i/o*/ register req * r,
     
     rc = EvalBool (r, (char *)sArg, (r -> CmdStack.State.pStart - r -> Buf.pBuf), &r -> CmdStack.State.nResult) ;
 
-    if (!r -> CmdStack.State.nResult && rc == ok) 
+    if (!r -> CmdStack.State.nResult && rc == ok && !r -> pImportStash)
         {
         r -> Buf.pCurrPos = r -> CmdStack.State.pStart ;        
         r -> Buf.nBlockNo = r -> CmdStack.State.nBlockNo ;        
@@ -622,7 +622,10 @@ static int CmdForeach (/*i/o*/ register req * r,
 	    if ((rc = EvalTransFlags (r, sVarName, (r -> CmdStack.State.pStart - r -> Buf.pBuf), G_ARRAY, &pRV)) != ok)
                 return rc ;
 
-            if (pRV == NULL)
+	    if (r -> pImportStash)
+		return ok ;
+          
+	    if (pRV == NULL)
                 return rcMissingArgs ;
 
             if (SvTYPE (pRV) != SVt_RV)
@@ -867,11 +870,14 @@ static int CmdVar (/*i/o*/ register req * r,
         return ok ;
     
     sv_setiv (*ppSV, 1) ;
-    
+
+    tainted = 0 ;
+
     pSV = newSVpvf("package %s ; \n#line %d %s\n use vars qw(%s); map { $%s::CLEANUP{substr ($_, 1)} = 1 } qw(%s) ;\n",
 	           r -> Buf.sEvalPackage, r -> Buf.nSourceline, r -> Buf.pFile -> sSourcefile, sArg,
 		   r -> Buf.sEvalPackage, sArg) ;
-    rc = EvalDirect (r, pSV) ;
+
+    rc = EvalDirect (r, pSV, 0, NULL) ;
     SvREFCNT_dec(pSV);
 
     return rc ;
@@ -1451,7 +1457,7 @@ static int HtmlTableHead (/*i/o*/ register req * r,
 /* ---------------------------------------------------------------------------- */
 
 
-static SV * SplitFdat     (/*i/o*/ register req * r,
+SV * SplitFdat     (/*i/o*/ register req * r,
                            /*in*/  SV ** ppSVfdat,
                            /*out*/ SV ** ppSVerg,
                            /*in*/  char * pName,
@@ -1631,7 +1637,7 @@ static int HtmlOption (/*i/o*/ register req * r,
                 oputc (r, ' ') ;
                 oputs (r, sArg) ;
                 }
-            oputs (r, " SELECTED>") ;
+            oputs (r, " selected>") ;
             r -> Buf.pCurrPos = NULL ; /* nothing more left of html tag */
             return ok ;
             }
@@ -1758,28 +1764,37 @@ static int HtmlInput (/*i/o*/ register req * r,
         if (vlen > 0 && ppSV)
             {
             SV * pSV ;
-            SV * * ppSVerg = hv_fetch(r -> pFormSplitHash, (char *)pName, nlen, 0) ;  
+            SV * pSVVal ;
+            char * pTVal ;
+	    STRLEN vtlen ;
+	    
+	    SV * * ppSVerg = hv_fetch(r -> pFormSplitHash, (char *)pName, nlen, 0) ;  
             pSV = SplitFdat (r, ppSV, ppSVerg, (char *)pName, nlen) ;
     
+	    pSVVal = newSVpv ((char *)pVal, vlen) ;
+	    TransHtmlSV (r, pSVVal) ;
+	    pTVal = SvPV (pSVVal, vtlen) ;
+
             if (SvTYPE (pSV) == SVt_PVHV)
                 { /* -> Hash -> check if key exists */
-                if (hv_exists ((HV *)pSV, (char *)pVal, vlen))
+                if (hv_exists ((HV *)pSV, (char *)pTVal, vtlen))
                     bEqual = 1 ;
                 }
             else
                 {
                 pData = SvPV (pSV, dlen) ;
-                if (dlen == vlen && strncmp (pVal, pData, dlen) == 0)
+                if (dlen == vtlen && strncmp (pTVal, pData, dlen) == 0)
                     bEqual = 1 ;
                 }
-            }
+            SvREFCNT_dec (pSVVal) ;
+	    }
        
-        pCheck = GetHtmlArg (sArg, "CHECKED", &clen) ;
+        pCheck = GetHtmlArg (sArg, "checked", &clen) ;
         if (pCheck)
             {
             if (!bEqual)
                 { /* Remove "checked" */
-                oputs (r, "<INPUT ") ;
+                oputs (r, "<input ") ;
                 
                 owrite (r, sArg, pCheck - sArg) ;
     
@@ -1793,9 +1808,9 @@ static int HtmlInput (/*i/o*/ register req * r,
             {
             if (bEqual)
                 { /* Insert "checked" */
-                oputs (r, "<INPUT ") ;
+                oputs (r, "<input ") ;
                 oputs (r, sArg) ;
-                oputs (r, " CHECKED>") ;
+                oputs (r, " checked>") ;
     
                 r -> Buf.pCurrPos = NULL ; /* nothing more left of html tag */
                 }
@@ -1805,11 +1820,11 @@ static int HtmlInput (/*i/o*/ register req * r,
         { /* text field */
         if (pVal)
             {
-            oputs (r, "<INPUT ") ;
+            oputs (r, "<input ") ;
 
             owrite (r, sArg, pVal - sArg) ;
 
-            oputs (r, " VALUE=\"") ;
+            oputs (r, " value=\"") ;
             OutputToHtml (r, pData) ;
             oputs (r, "\" ") ;
 
@@ -1823,9 +1838,9 @@ static int HtmlInput (/*i/o*/ register req * r,
             }
         else
             {
-            oputs (r, "<INPUT ") ;
+            oputs (r, "<input ") ;
             oputs (r, sArg) ;
-            oputs (r, " VALUE=\"") ;
+            oputs (r, " value=\"") ;
             OutputToHtml (r, pData) ;
             oputs (r, "\">") ;
     
