@@ -26,11 +26,15 @@
 *
 ------------------------------------------------------------------------------- */
 
-int EvalDirect (/*i/o*/ register req * r,
-			/*in*/    SV * pArg) 
+int EvalDirect (/*i/o*/ register req *  r,
+		/*in*/  SV *            pArg, 
+                /*in*/  int		numArgs,
+                /*in*/  SV **		pArgs)
     {
+    dTHXsem 
     dSP;
     SV *  pSVErr  ;
+    int   num ;         
 
     EPENTRY (EvalDirect) ;
 
@@ -38,6 +42,10 @@ int EvalDirect (/*i/o*/ register req * r,
     pCurrReq = r ;
 
     PUSHMARK(sp);
+    for (num = 0; num < numArgs; num++)
+	XPUSHs(pArgs [num]) ;            /* push pointer to argument */
+    PUTBACK;
+
     perl_eval_sv(pArg, G_SCALAR | G_KEEPERR);
 
 
@@ -63,6 +71,95 @@ int EvalDirect (/*i/o*/ register req * r,
     }
 
 
+/* -------------------------------------------------------------------------------
+*
+* Eval Config Statements 
+* 
+* in  pSV    pointer to string or CV
+* out pCV    pointer to SV contains an CV to the evaled code
+*
+------------------------------------------------------------------------------- */
+
+int EvalConfig (/*i/o*/ register req *  r,
+		/*in*/  SV *            pSV, 
+                /*in*/  int		numArgs,
+                /*in*/  SV **		pArgs,
+		/*out*/ CV **           pCV)
+    {
+    dTHXsem 
+    dSP;
+    SV *  pSVErr  ;
+    int   num ;         
+    char * s = "Needs CodeRef" ;
+
+    EPENTRY (EvalDirect) ;
+
+    tainted = 0 ;
+    pCurrReq = r ;
+
+    *pCV = NULL ;
+    if (SvPOK (pSV))
+	{
+	STRLEN l ;
+	s = SvPV (pSV, l) ;
+	if (strncmp (s, "sub ", 4) == 0)
+	    {
+	    SV * pSVErr ;
+	    SV * pRV ;
+
+	    pRV = perl_eval_pv (s, 0) ;
+	    if (SvROK (pRV))
+		{
+		*pCV = (CV *)SvRV (pRV) ;
+		SvREFCNT_inc (*pCV) ;
+		}
+
+	    pSVErr = ERRSV ;
+	    if (SvTRUE (pSVErr))
+		{
+		STRLEN l ;
+		char * p = SvPV (pSVErr, l) ;
+		if (l > sizeof (r -> errdat1) - 1)
+		    l = sizeof (r -> errdat1) - 1 ;
+		strncpy (r -> errdat1, p, l) ;
+		if (l > 0 && r -> errdat1[l-1] == '\n')
+		    l-- ;
+		r -> errdat1[l] = '\0' ;
+         
+		LogError (r, rcEvalErr) ;
+
+		sv_setpv(pSVErr,"");
+		*pCV = NULL ;
+		return rcEvalErr ;
+		}
+	    }
+	else
+	    {
+	    *pCV = perl_get_cv (s, 0) ;
+	    SvREFCNT_inc (*pCV) ;
+	    }
+	}
+    else 
+	{
+	if (SvROK (pSV))
+	    {
+	    *pCV = (CV *)SvRV (pSV) ;
+	    }
+	}
+
+    if (!*pCV || SvTYPE (*pCV) != SVt_PVCV)
+	{
+	*pCV = NULL ;
+	strcpy (r -> errdat1 ,"Config: ") ;
+	strncpy (r -> errdat2, s, sizeof (r -> errdat2) - 1) ;
+	return rcEvalErr ;
+	}
+
+    return ok ;
+    }
+
+
+    
 
 /* -------------------------------------------------------------------------------
 *
@@ -79,6 +176,7 @@ static int EvalAll (/*i/o*/ register req * r,
                     /*in*/  const char *  sName,
 		    /*out*/ SV **         pRet)             
     {
+    dTHXsem 
     static char sFormat []       = "package %s ; sub %s { \n#line %d \"%s\"\n%s\n} %s%s" ;
     static char sFormatStrict [] = "package %s ; use strict ; sub %s {\n#line %d \"%s\"\n%s\n} %s%s" ; 
     static char sFormatArray []       = "package %s ; sub %s { \n#line %d \"%s\"\n[%s]\n} %s%s" ;
@@ -95,7 +193,7 @@ static int EvalAll (/*i/o*/ register req * r,
     GetLineNo (r) ;
 
     if (r -> bDebug & dbgDefEval)
-        lprintf (r, "[%d]DEF:  Line %d: %s\n", r -> nPid, r -> Buf.nSourceline, sArg) ;
+        lprintf (r, "[%d]DEF:  Line %d: %s\n", r -> nPid, r -> Buf.nSourceline, sArg?sArg:"<unknown>") ;
 
     tainted = 0 ;
     pCurrReq = r ;
@@ -169,6 +267,7 @@ static int EvalAllNoCache (/*i/o*/ register req * r,
 			/*in*/  const char *  sArg,
                            /*out*/ SV **         pRet)             
     {
+    dTHXsem 
     int   num ;         
     int   nCountUsed = r -> TableStack.State.nCountUsed ;
     int   nRowUsed   = r -> TableStack.State.nRowUsed ;
@@ -182,7 +281,7 @@ static int EvalAllNoCache (/*i/o*/ register req * r,
     EPENTRY (EvalAll) ;
 
     if (r -> bDebug & dbgEval)
-        lprintf (r, "[%d]EVAL< %s\n", r -> nPid, sArg) ;
+        lprintf (r, "[%d]EVAL< %s\n", r -> nPid, sArg?sArg:"<unknown>") ;
 
     tainted = 0 ;
     pCurrReq = r ;
@@ -296,24 +395,32 @@ static int Watch (/*i/o*/ register req * r)
 ------------------------------------------------------------------------------- */
 
 
-static int CallCV  (/*i/o*/ register req * r,
+int CallCV  (/*i/o*/ register req * r,
 		    /*in*/  const char *  sArg,
                     /*in*/  CV *          pSub,
                     /*in*/  int           flags,
                     /*out*/ SV **         pRet)             
     {
+    dTHXsem 
     int   num ;         
     int   nCountUsed = r -> TableStack.State.nCountUsed ;
     int   nRowUsed   = r -> TableStack.State.nRowUsed ;
     int   nColUsed   = r -> TableStack.State.nColUsed ;
     SV *  pSVErr ;
-
     dSP;                            /* initialize stack pointer      */
 
+
+    if (r -> pImportStash)
+	{ /* do not execute any code on import */
+	*pRet = NULL ;
+	return ok ;
+	}
+    
+
     EPENTRY (CallCV) ;
 
     if (r -> bDebug & dbgEval)
-        lprintf (r, "[%d]EVAL< %s\n", r -> nPid, sArg) ;
+        lprintf (r, "[%d]EVAL< %s\n", r -> nPid, sArg?sArg:"<unknown>") ;
 
     tainted = 0 ;
     pCurrReq = r ;
@@ -448,7 +555,7 @@ static int CallCV  (/*i/o*/ register req * r,
 ------------------------------------------------------------------------------- */
 
 
-static int EvalOnly (/*i/o*/ register req * r,
+int EvalOnly           (/*i/o*/ register req * r,
 			/*in*/  const char *  sArg,
                         /*in*/  SV **         ppSV,
                         /*in*/  int           flags,
@@ -564,6 +671,205 @@ static int EvalAndCall (/*i/o*/ register req * r,
     return rcEvalErr ;
     }
 
+#ifdef EP2
+
+/* -------------------------------------------------------------------------------
+*
+* Call an already evaled PERL Statement
+* 
+* in  sArg   Statement to eval (only used for logging)
+* in  pSub   CV which should be called
+* in  numArgs number of arguments
+* in  pArgs   args for subroutine
+* out pRet   pointer to SV contains the eval return
+*
+------------------------------------------------------------------------------- */
+
+
+int CallStoredCV  (/*i/o*/ register req * r,
+		    /*in*/  const char *  sArg,
+                    /*in*/  CV *          pSub,
+                    /*in*/  int           numArgs,
+                    /*in*/  SV **         pArgs,
+                    /*in*/  int           flags,
+                    /*out*/ SV **         pRet)             
+    {
+    int   num ;         
+    SV *  pSVErr ;
+
+    dSP;                            /* initialize stack pointer      */
+
+    EPENTRY (CallCV) ;
+
+    if (r -> bDebug & dbgEval)
+        lprintf (r, "[%d]EVAL< %s\n", r -> nPid, sArg?sArg:"<unknown>") ;
+
+    tainted = 0 ;
+    pCurrReq = r ;
+
+    ENTER ;
+    SAVETMPS ;
+    PUSHMARK(sp);                   /* remember the stack pointer    */
+    for (num = 0; num < numArgs; num++)
+	XPUSHs(pArgs [num]) ;            /* push pointer to argument */
+    PUTBACK;
+
+    num = perl_call_sv ((SV *)pSub, flags | G_EVAL | (numArgs?0:G_NOARGS)) ; /* call the function             */
+    
+    SPAGAIN;                        /* refresh stack pointer         */
+    
+    if (r -> bDebug & dbgMem)
+        lprintf (r, "[%d]SVs:  %d\n", r -> nPid, sv_count) ;
+    /* pop the return value from stack */
+    if (num == 1)   
+        {
+        *pRet = POPs ;
+        if (SvTYPE (*pRet) == SVt_PVMG)
+            { /* variable is magicaly -> fetch value now */
+            SV * pSV = newSVsv (*pRet) ;
+            *pRet = pSV ;
+            }
+        else        
+            SvREFCNT_inc (*pRet) ;
+
+        if (r -> bDebug & dbgEval)
+            {
+            if (SvOK (*pRet))
+                lprintf (r, "[%d]EVAL> %s\n", r -> nPid, SvPV (*pRet, na)) ;
+            else
+                lprintf (r, "[%d]EVAL> <undefined>\n", r -> nPid) ;
+            }                
+        }
+     else if (num == 0)
+        {
+        *pRet = NULL ;
+        if (r -> bDebug & dbgEval)
+            lprintf (r, "[%d]EVAL> <NULL>\n", r -> nPid) ;
+        }
+     else
+        {
+        *pRet = &sv_undef ;
+        if (r -> bDebug & dbgEval)
+            lprintf (r, "[%d]EVAL> returns %d args instead of one\n", r -> nPid, num) ;
+        }
+
+     PUTBACK;
+     FREETMPS ;
+     LEAVE ;
+
+     if (r -> bExit)
+	 {
+	 if (*pRet)
+	     SvREFCNT_dec (*pRet) ;
+	 *pRet = NULL ;
+	 return rcExit ;
+	 }
+     
+     pSVErr = ERRSV ;
+     if (SvTRUE (pSVErr))
+        {
+        STRLEN l ;
+        char * p ;
+
+        if (SvMAGICAL (pSVErr) && mg_find (pSVErr, 'U'))
+            {
+ 	    /* On an Apache::exit call, the function croaks with error having 'U' magic.
+ 	     * When we get this return, we'll just give up and quit this file completely,
+ 	     * without error. */
+             
+	    /*struct magic * m = SvMAGIC (pSVErr) ;*/
+
+	    sv_unmagic(pSVErr,'U');
+	    sv_setpv(pSVErr,"");
+
+	    r -> bOptions |= optNoUncloseWarn ;
+	    r -> bExit = 1 ;
+
+            return rcExit ;
+            }
+
+        p = SvPV (pSVErr, l) ;
+        if (l > sizeof (r -> errdat1) - 1)
+            l = sizeof (r -> errdat1) - 1 ;
+        strncpy (r -> errdat1, p, l) ;
+        if (l > 0 && r -> errdat1[l-1] == '\n')
+             l-- ;
+        r -> errdat1[l] = '\0' ;
+         
+	LogError (r, rcEvalErr) ;
+
+	sv_setpv(pSVErr,"");
+
+	return rcEvalErr ;
+        }
+
+     
+    return ok ;
+    }
+
+
+/* -------------------------------------------------------------------------------
+*
+* Eval PERL Statements check if it's already compiled
+* 
+* in  sArg      Statement to eval
+* in  nFilepos  position von eval in file (is used to build an unique key)
+* out pRet      pointer to SV contains the eval return
+*
+------------------------------------------------------------------------------- */
+
+int EvalStore (/*i/o*/ register req * r,
+	      /*in*/  const char *  sArg,
+	      /*in*/  int           nFilepos,
+	      /*out*/ SV **         pRet)             
+
+
+    {
+    int     rc ;
+    SV **   ppSV ;
+    
+    
+    EPENTRY (Eval) ;
+
+    r -> numEvals++ ;
+    *pRet = NULL ;
+
+    /*if (r -> bDebug & dbgCacheDisable)
+        return EvalAllNoCache (r, sArg, pRet) ;
+    */
+    /* Already compiled ? */
+
+    ppSV = hv_fetch(r -> Buf.pFile -> pCacheHash, (char *)&nFilepos, sizeof (nFilepos), 1) ;  
+    if (ppSV == NULL)
+        return rcHashError ;
+
+    if (*ppSV != NULL && SvTYPE (*ppSV) == SVt_PV)
+        {
+        strncpy (r -> errdat1, SvPV(*ppSV, na), sizeof (r -> errdat1) - 1) ; 
+        LogError (r, rcEvalErr) ;
+        return rcEvalErr ;
+        }
+
+    lprintf (r, "CV ppSV=%s type=%d\n", *ppSV?"ok":"NULL", *ppSV?SvTYPE (*ppSV):0) ;               
+    if (*ppSV == NULL || SvTYPE (*ppSV) != SVt_PVCV)
+	{
+	if ((rc = EvalOnly (r, sArg, ppSV, G_SCALAR, "")) != ok)
+	    {
+	    *pRet = NULL ;
+	    return rc ;
+	    }
+        *pRet = *ppSV  ;
+	return ok ;
+	}
+
+    *pRet = *ppSV  ;
+    r -> numCacheHits++ ;
+    return ok ;
+    }
+
+
+
+#endif /* EP2 */
 
 /* -------------------------------------------------------------------------------
 *
@@ -590,9 +896,9 @@ int Eval (/*i/o*/ register req * r,
     r -> numEvals++ ;
     *pRet = NULL ;
 
-    if (r -> bDebug & dbgCacheDisable)
+    /*if (r -> bDebug & dbgCacheDisable)
         return EvalAllNoCache (r, sArg, pRet) ;
-
+    */
     /* Already compiled ? */
 
     ppSV = hv_fetch(r -> Buf.pFile -> pCacheHash, (char *)&nFilepos, sizeof (nFilepos), 1) ;  
@@ -642,14 +948,15 @@ int EvalTransFlags (/*i/o*/ register req * r,
     r -> numEvals++ ;
     *pRet = NULL ;
 
+    /*
     if (r -> bDebug & dbgCacheDisable)
         {
-        /* strip off all <HTML> Tags */
+        /  * strip off all <HTML> Tags *  /
         TransHtml (r, sArg, 0) ;
         
         return EvalAllNoCache (r, sArg, pRet) ;
         }
-
+    */
     /* Already compiled ? */
 
     ppSV = hv_fetch(r -> Buf.pFile -> pCacheHash, (char *)&nFilepos, sizeof (nFilepos), 1) ;  
@@ -730,11 +1037,19 @@ int EvalTransOnFirstCall (/*i/o*/ register req * r,
 
     if (*ppSV == NULL || SvTYPE (*ppSV) != SVt_PVCV)
         {
-        /* strip off all <HTML> Tags */
+        int	rc ;
+	HV *  pImportStash = r -> pImportStash ;
+	r -> pImportStash = NULL ; /* temporarely disable import */
+
+	/* strip off all <HTML> Tags */
         TransHtml (r, sArg, 0) ;
 
-        return EvalAndCall (r, sArg, ppSV, G_SCALAR, pRet) ;
-        }
+	rc = EvalAndCall (r, sArg, ppSV, G_SCALAR, pRet) ;
+
+       	r -> pImportStash = pImportStash ; 
+
+	return rc ;	    
+	}
 
     r -> numCacheHits++ ;
     
@@ -986,7 +1301,7 @@ int EvalMain (/*i/o*/ register req *  r)
 	    pClose = NULL ;
 	    if (pOpen)
                 {
-                if ((pClose = strstr (pOpen, pCloseBracket)) == NULL)
+		if ((pClose = strstr (pOpen + lenOpenBracket, pCloseBracket)) == NULL)
                     {
                     strncpy (r -> errdat1, pCloseBracket, sizeof (r -> errdat1) - 1) ; 
                     return rcMissingRight ;
