@@ -24,6 +24,7 @@
 #define oCommitToMem           EMBPERL_oCommitToMem
 #define OpenInput              EMBPERL_OpenInput         
 #define CloseInput             EMBPERL_CloseInput        
+#define ReadInputFile          EMBPERL_ReadInputFile        
 #define iread                  EMBPERL_iread             
 #define igets                  EMBPERL_igets             
 #define OpenOutput             EMBPERL_OpenOutput        
@@ -40,9 +41,11 @@
 #define _malloc                EMBPERL__malloc           
 #define LogError               EMBPERL_LogError          
 #define OutputToHtml           EMBPERL_OutputToHtml      
+#define OutputEscape           EMBPERL_OutputEscape      
 #define Eval                   EMBPERL_Eval              
 #define EvalNum                EMBPERL_EvalNum           
 #define EvalBool               EMBPERL_EvalBool           
+#define EvalConfig             EMBPERL_EvalConfig
 #define stristr                EMBPERL_stristr           
 #define strlower               EMBPERL_strlower          
 #define TransHtml              EMBPERL_TransHtml         
@@ -50,6 +53,8 @@
 #define GetHtmlArg             EMBPERL_GetHtmlArg        
 #define GetHashValueLen        EMBPERL_GetHashValueLen   
 #define GetHashValue           EMBPERL_GetHashValue      
+#define GetHashValueStrDup     EMBPERL_GetHashValueStrDup      
+#define SetHashValueStr        EMBPERL_SetHashValueStr      
 #define Char2Html              EMBPERL_Char2Html         
 #define Html2Char              EMBPERL_Html2Char         
 #define sizeHtml2Char          EMBPERL_sizeHtml2Char     
@@ -66,12 +71,15 @@
 #define EvalTransFlags         EMBPERL_EvalTransFlags
 #define EvalTransOnFirstCall   EMBPERL_EvalTransOnFirstCall           
 #define EvalSub                EMBPERL_EvalSub
+#define EvalOnly               EMBPERL_EvalOnly
+#define CallCV                 EMBPERL_CallCV
 #define GetContentLength       EMBPERL_GetContentLength    
 #define GetLogFilePos          EMBPERL_GetLogFilePos       
 #define ReadHTML               EMBPERL_ReadHTML            
 #define ScanCmdEvalsInString   EMBPERL_ScanCmdEvalsInString
 #define EvalDirect             EMBPERL_EvalDirect
 #define GetLineNo              EMBPERL_GetLineNo
+#define GetLineNoOf            EMBPERL_GetLineNoOf
 #define Dirname                EMBPERL_Dirname
 #define CommitError            EMBPERL_CommitError
 #define RollbackError          EMBPERL_RollbackError
@@ -91,12 +99,15 @@
 #define SetupRequest           EMBPERL_SetupRequest   
 #define Term                   EMBPERL_Term           
 #define sstrdup                EMBPERL_sstrdup        
+#define _ep_memdup             EMBPERL__ep_memdup
 #define ProcessBlock           EMBPERL_ProcessBlock
 #define NewEscMode             EMBPERL_NewEscMode
 #define GetSubTextPos          EMBPERL_GetSubTextPos
 #define SetSubTextPos          EMBPERL_SetSubTextPos
 #define SetupDebugger          EMBPERL_SetupDebugger
-
+#define GetFileData            EMBPERL_GetFileData
+#define SplitFdat              EMBPERL_SplitFdat
+#define AddMagicAV             EMBPERL_AddMagicAV
 
 #define InitialReq             EMBPERL_InitialReq
 #define pCurrReq               EMBPERL_pCurrReq
@@ -107,6 +118,25 @@
 #define PERL_SUBVERSION SUBVERSION
 #endif
 
+#ifndef pTHX_
+#define pTHX_
+#endif
+#ifndef pTHX
+#define pTHX
+#endif
+#ifndef aTHX_
+#define aTHX_
+#endif
+#ifndef aTHX
+#define aTHX
+#endif
+#ifndef dTHX
+#define dTHX
+#define dTHXsem
+#else
+#define dTHXsem dTHX ;
+#endif
+
 
 #if PERL_VERSION >= 5
 
@@ -204,3 +234,71 @@
 
 
 #endif /* PERL_VERSION > 5 */
+
+
+#ifdef APACHE
+
+#ifdef WIN32
+
+#undef uid_t
+#ifdef apache_uid_t
+#define uid_t apache_uid_t
+#undef apache_uid_t
+#endif
+
+#undef gid_t
+#ifdef apache_gid_t
+#define gid_t apache_gid_t
+#undef apache_gid_t
+#endif
+
+#undef mode_t
+#ifdef apache_mode_t
+#define gid_t apache_mode_t
+#undef apache_mode_t
+#endif
+
+#ifdef apache_stat
+#undef stat
+#define stat apache_stat
+#undef apache_stat
+#endif
+
+#ifdef apache_sleep
+#undef sleep
+#define sleep apache_sleep
+#undef apache_sleep
+#endif
+
+#if PERL_VERSION >= 6
+
+#ifdef apache_opendir
+#undef opendir
+#define opendir apache_opendir
+#undef apache_opendir
+#endif
+
+#ifdef apache_readdir
+#undef readdir
+#define readdir apache_readdir
+#undef apache_readdir
+#endif
+
+#ifdef apache_closedir
+#undef closedir
+#define closedir apache_closedir
+#undef apache_closedir
+#endif
+
+#ifdef apache_crypt
+#undef crypt
+#define crypt apache_crypt
+#undef apache_crypt
+#endif
+
+#endif /* endif PERL_IS_5_6 */
+
+#endif /* endif WIN32 */
+
+#endif /* APACHE */
+
