@@ -44,7 +44,7 @@ void OutputToHtml (/*i/o*/ register req * r,
     
     while (*sData)
         {
-        if (*sData == '\\')
+        if (*sData == '\\' && (r -> nCurrEscMode & escEscape) == 0)
             {
             if (p != sData)
                 owrite (r, p, sData - p) ;
@@ -68,6 +68,65 @@ void OutputToHtml (/*i/o*/ register req * r,
         owrite (r, p, sData - p) ;
     }
 
+/* ---------------------------------------------------------------------------- */
+/*                                                                              */
+/* Output a string and escape it                                                */
+/*                                                                              */
+/* in sData     = input:  string                                                */
+/*    nDataLen  = input:  length of string                                      */
+/*    pEscTab   = input:  escape table                                          */
+/*    cEscChar  = input:  char to escape escaping (0 = off)                     */
+/*                                                                              */
+/* ---------------------------------------------------------------------------- */
+
+void OutputEscape (/*i/o*/ register req * r,
+ 		   /*in*/  const char *   sData,
+ 		   /*in*/  int            nDataLen,
+ 		   /*in*/  struct tCharTrans *   pEscTab,
+ 		   /*in*/  char           cEscChar)
+
+    {
+    char * pHtml  ;
+    const char * p ;
+    int	         l ;
+
+    EPENTRY (OutputEscape) ;
+
+    if (pEscTab == NULL)
+        {
+        owrite (r, sData, nDataLen) ;
+        return ;
+        }
+
+    p = sData ;
+    l = nDataLen ;
+
+    while (l > 0)
+        {
+        if (cEscChar && *sData == cEscChar)
+            {
+            if (p != sData)
+                owrite (r, p, sData - p) ;
+            sData++, l-- ;
+            p = sData ;
+            }
+        else
+            {
+            pHtml = pEscTab[(unsigned char)(*sData)].sHtml ;
+            if (*pHtml)
+                {
+                if (p != sData)
+                    owrite (r, p, sData - p) ;
+                oputs (r, pHtml) ;
+                p = sData + 1;
+                }
+            }
+        sData++, l-- ;
+        }
+    if (p != sData)
+        owrite (r, p, sData - p) ;
+    }
+
 
 #if 0
 
@@ -198,14 +257,19 @@ int   TransHtml (/*i/o*/ register req * r,
     EPENTRY (TransHtml) ;
 	
     if (r -> bOptions & optRawInput)
-	{ /* Just remove CR for raw input */
-	while (*p != '\0')
+	{ 
+#if PERL_VERSION < 5
+	/* Just remove CR for raw input for perl 5.004 */
+	if (nLen == 0)
+	    nLen = strlen (sData) ;
+	e = sData + nLen ;
+	while (p < e)
 	    {
 	    if (*p == '\r')
 	    	*p = ' ' ;
 	    p++ ;
 	    }	
-
+#endif
 	return nLen ; 	
         }
         
@@ -214,7 +278,7 @@ int   TransHtml (/*i/o*/ register req * r,
         nLen = strlen (sData) ;
     e = sData + nLen ;
 
-    while (*p && p < e)
+    while (p < e)
 	{
 	if (*p == '\\')
 	    {
@@ -224,7 +288,7 @@ int   TransHtml (/*i/o*/ register req * r,
 		memmove (p, p + 1, e - p - 1) ;
 		e[-1] = ' ' ;
 		p++ ;
-		while (*p && *p != '>')
+		while (p < e && *p != '>')
 		    p++ ;
 		}
 	    else if (p[1] == '&')
@@ -232,7 +296,7 @@ int   TransHtml (/*i/o*/ register req * r,
 		memmove (p, p + 1, e - p - 1) ;
 		e[-1] = ' ' ;
 		p++ ;
-		while (*p && *p != ';')
+		while (p < e && *p != ';')
 		    p++ ;
 		}
 	    else if (bInUrl && p[1] == '%')
@@ -244,19 +308,22 @@ int   TransHtml (/*i/o*/ register req * r,
 	    else
 		p++ ; /* Nothing to quote */
 	    }
+#if PERL_VERSION < 5
+	/* remove CR for perl 5.004 */
 	else if (*p == '\r')
 	    {
 	    *p++ = ' ' ;
 	    }
+#endif
 	else
 	    {
 	    if (p[0] == '<' && (isalpha (p[1]) || p[1] == '/'))
 		{ /*  count HTML tag length */
 		s = p ;
 		p++ ;
-		while (*p && *p != '>')
+		while (p < e && *p != '>')
 		    p++ ;
-		if (*p)
+		if (p < e)
 		    p++ ;
 		else
 		    { /* missing left '>' -> no html tag  */
@@ -268,10 +335,10 @@ int   TransHtml (/*i/o*/ register req * r,
 		{ /*  count HTML char length */
 		s = p ;
 		p++ ;
-		while (*p && *p != ';')
+		while (p < e && *p != ';')
 		    p++ ;
 
-		if (*p)
+		if (p < e)
 		    {
 		    *p = '\0' ;
 		    p++ ;
@@ -308,7 +375,7 @@ int   TransHtml (/*i/o*/ register req * r,
 		s = NULL ;
 		}
 	    else
-		if (*p)
+		if (p < e)
 		    p++ ;
 	    }
 	}
@@ -481,9 +548,9 @@ char * GetHashValue (/*in*/  HV *           pHash,
 
 
 
-int    GetHashValueInt (/*in*/  HV *           pHash,
+IV    GetHashValueInt (/*in*/  HV *           pHash,
                         /*in*/  const char *   sKey,
-                        /*in*/  int            nDefault)
+                        /*in*/  IV            nDefault)
 
     {
     SV **   ppSV ;
@@ -504,16 +571,56 @@ char * GetHashValueStr (/*in*/  HV *           pHash,
 
     {
     SV **   ppSV ;
+    STRLEN  l ;
 
     /*EPENTRY (GetHashValueInt) ;*/
 
     ppSV = hv_fetch(pHash, (char *)sKey, strlen (sKey), 0) ;  
     if (ppSV != NULL)
-        return SvPV (*ppSV, na) ;
+        return SvPV (*ppSV, l) ;
         
     return sDefault ;
     }
 
+char * GetHashValueStrDup (/*in*/  HV *           pHash,
+                           /*in*/  const char *   sKey,
+                           /*in*/  char *         sDefault)
+    {
+    SV **   ppSV ;
+    STRLEN  l ;
+    char *  s ;
+
+    ppSV = hv_fetch(pHash, (char *)sKey, strlen (sKey), 0) ;  
+    if (ppSV != NULL)
+        {
+	if (s = SvPV (*ppSV, l))
+	    return strdup (s);
+	else
+	    return NULL ;
+	}
+
+    if (sDefault)
+        return strdup (sDefault) ;
+    else
+	return NULL ;
+    }
+
+
+void SetHashValueStr   (/*in*/  HV *           pHash,
+                        /*in*/  const char *   sKey,
+                        /*in*/  char *         sValue)
+
+    {
+    SV *   pSV = newSVpv (sValue, 0) ;
+
+    /*EPENTRY (GetHashValueInt) ;*/
+
+    hv_store(pHash, (char *)sKey, strlen (sKey), pSV, 0) ;  
+    }
+
+
+
+
 
 /* ------------------------------------------------------------------------- */
 /*                                                                           */
@@ -527,10 +634,11 @@ char * GetHashValueStr (/*in*/  HV *           pHash,
 /* ------------------------------------------------------------------------- */
 
 
-int GetLineNo (/*i/o*/ register req * r)
+int GetLineNoOf (/*i/o*/ register req * r,
+               /*in*/  char * pPos)
 
     {
-    char * pPos = r -> Buf.pCurrPos ;
+
     
     if (r -> Buf.pSourcelinePos == NULL)
 	if (r -> Buf.pFile == NULL)
@@ -571,6 +679,14 @@ int GetLineNo (/*i/o*/ register req * r)
     }
 
 
+int GetLineNo (/*i/o*/ register req * r)
+
+    {
+    char * pPos = r -> Buf.pCurrPos ;
+    
+    return GetLineNoOf (r, pPos) ;
+    }
+
 
 /* ------------------------------------------------------------------------- */
 /*                                                                           */
